package model

import (
	"ArmedPolice/app/common/model"
	"ArmedPolice/serve/orm"
	"fmt"
	"time"
)

type WorkInstance struct {
	*model.WorkInstance
}

// WorkInstanceInfo 基本信息
type WorkInstanceInfo struct {
	ID             uint64    `json:"-"`
	Title          string    `json:"title"`
	EquipmentCode  string    `json:"equipment_code"`
	EquipmentTitle string    `json:"equipment_title"`
	BreakdownTitle string    `json:"breakdown_title"`
	Priority       int       `json:"priority"`
	Status         int       `json:"status"`
	CreatedAt      time.Time `json:"created_at"`
}

// WorkbenchCondition 工作台条件
type WorkbenchCondition struct {
	UID     string
	RoleIDs []string
}

// Instances 基本信息
func (m *WorkInstance) Instances(page, pageSize int, count *int64, where ...*model.ModelWhere) ([]*WorkInstanceInfo, error) {
	db := orm.GetDB().Table(m.TableName()+" AS w").
		Select("w.id", "w.title", "e.code AS equipment_code", "e.title AS equipment_title", "w.priority",
			"(SELECT GROUP_CONCAT(s_b.title) FROM ( SELECT id, title FROM sys_breakdown) AS s_b WHERE FIND_IN_SET( s_b.id, w.breakdown )) AS breakdown_title",
			"w.status", "w.created_at").
		Joins(fmt.Sprintf("LEFT JOIN %s AS e ON w.equipment_id = e.id", model.NewManageEquipment().TableName())).
		Where("w.is_deleted = ?", model.DeleteStatusForNot)

	if len(where) > 0 {
		for _, wo := range where {
			db = db.Where(wo.Condition, wo.Value)
		}
	}
	out := make([]*WorkInstanceInfo, 0)

	if err := db.Count(count).Error; err != nil {
		return nil, err
	}

	if err := db.Order("w.id " + model.OrderModeToDesc).Offset((page - 1) * pageSize).Limit(pageSize).Scan(&out).Error; err != nil {
		return nil, err
	}
	return out, nil
}

// Persons 个人信息
func (m *WorkInstance) Persons(uid uint64, kind model.WorkScheduleKind, page, pageSize int, count *int64, where ...*model.ModelWhere) ([]*WorkInstanceInfo, error) {
	mWorkSchedule := model.NewWorkSchedule()

	db := orm.GetDB().Table(model.NewWorkProgress().TableName()+" AS p").
		Select("w.id", "w.title", "e.code AS equipment_code", "e.title AS equipment_title", "w.priority",
			"(SELECT GROUP_CONCAT(s_b.title) FROM ( SELECT id, title FROM sys_breakdown) AS s_b WHERE FIND_IN_SET( s_b.id, w.breakdown )) AS breakdown_title",
			"w.status", "w.created_at").
		Joins(fmt.Sprintf("LEFT JOIN %s AS w ON p.work_id = w.id", m.TableName())).
		Joins(fmt.Sprintf("LEFT JOIN %s AS e ON w.equipment_id = e.id", model.NewManageEquipment().TableName())).
		Joins(fmt.Sprintf("LEFT JOIN (SELECT id FROM %s WHERE kind = %d ORDER BY stage ASC, step ASC limit 1) AS s ON p.schedule_id = s.id",
			mWorkSchedule.TableName(), kind)).
		Where("p.is_deleted = ? AND w.is_deleted = ?", model.DeleteStatusForNot, model.DeleteStatusForNot).
		Where("p.uid = ?", uid).
		Where("s.id > ?", 0).
		Group("w.id")

	if len(where) > 0 {
		for _, wo := range where {
			db = db.Where(wo.Condition, wo.Value)
		}
	}
	out := make([]*WorkInstanceInfo, 0)

	if err := db.Count(count).Error; err != nil {
		return nil, err
	}
	if err := db.Order("w.id " + model.OrderModeToDesc).Offset((page - 1) * pageSize).Limit(pageSize).Scan(&out).Error; err != nil {
		return nil, err
	}
	return out, nil
}

// Workbench 个人信息
func (m *WorkInstance) Workbench(condition *WorkbenchCondition, page, pageSize int, count *int64, where ...*model.ModelWhere) ([]*WorkInstanceInfo, error) {
	db := orm.GetDB().Table(m.TableName()+" AS w").
		Select("w.id", "w.title", "e.code AS equipment_code", "e.title AS equipment_title", "w.priority",
			"(SELECT GROUP_CONCAT(s_b.title) FROM ( SELECT id, title FROM sys_breakdown) AS s_b WHERE FIND_IN_SET( s_b.id, w.breakdown )) AS breakdown_title",
			"w.status", "w.created_at").
		Joins(fmt.Sprintf("LEFT JOIN %s AS e ON w.equipment_id = e.id", model.NewManageEquipment().TableName())).
		Where("w.is_deleted = ?", model.DeleteStatusForNot)

	if len(where) > 0 {
		for _, wo := range where {
			db = db.Where(wo.Condition, wo.Value)
		}
	}
	out := make([]*WorkInstanceInfo, 0)

	//if err := orm.GetDB().Table(mWorkProgress.TableName()).Where("is_deleted = ?", model.DeleteStatusForNot).Count(count).Error; err != nil {
	//	return nil, err
	//}
	if err := db.Order("w.id " + model.OrderModeToDesc).Offset((page - 1) * pageSize).Limit(pageSize).Scan(&out).Error; err != nil {
		return nil, err
	}
	return out, nil
}

func NewWorkInstance() *WorkInstance {
	return &WorkInstance{model.NewWorkInstance()}
}
